#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/

void de_jac_trap_up_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[0] = 0.5*Dt*p[18] + 1; 
data[39] = 0.5*Dt; 
data[714] = -p[1]; 
data[721] = 1.0*p[0]; 
data[756] = -p[1]; 
data[763] = 1.0*p[0]; 
data[798] = -p[1]; 
data[805] = 1.0*p[0]; 
data[840] = -p[3]; 
data[847] = 1.0*p[2]; 
data[882] = p[5]/(pow(p[5], 2) + 1.0*pow(p[4], 2)); 
data[889] = 1.0*p[4]/(pow(p[5], 2) + 1.0*pow(p[4], 2)); 
data[919] = -1.0*p[0]; 
data[926] = -1.0*p[1]; 
data[961] = -1.0*p[0]; 
data[968] = -1.0*p[1]; 
data[1003] = -1.0*p[0]; 
data[1010] = -1.0*p[1]; 
data[1045] = -1.0*p[2]; 
data[1052] = -1.0*p[3]; 
data[1087] = -1.0*p[4]/(pow(p[5], 2) + 1.0*pow(p[4], 2)); 
data[1094] = 1.0*p[5]/(pow(p[5], 2) + 1.0*pow(p[4], 2)); 
data[1170] = 1.0/u[0]; 
data[1302] = -p[11]; 
data[1306] = 1.0*p[10]; 
data[1344] = -p[11]; 
data[1348] = 1.0*p[10]; 
data[1386] = -p[11]; 
data[1390] = 1.0*p[10]; 
data[1428] = -p[13]; 
data[1432] = 1.0*p[12]; 
data[1466] = -1.0*p[10]; 
data[1470] = -1.0*p[11]; 
data[1508] = -1.0*p[10]; 
data[1512] = -1.0*p[11]; 
data[1550] = -1.0*p[10]; 
data[1554] = -1.0*p[11]; 
data[1592] = -1.0*p[12]; 
data[1596] = -1.0*p[13]; 
data[1640] = p[17]; 
data[1679] = -p[16]; 

}

void de_jac_trap_xy_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[1124] = 2*p[7]*y[16] + p[8]*y[16]/sqrt(pow(y[23], 2) + pow(y[16], 2) + 9.9999999999999995e-7) + 0.40824829046386307*u[1]*u[0]*cos(u[5] + u[6]); 
data[1125] = 2*p[7]*y[17] + p[8]*y[17]/sqrt(pow(y[24], 2) + pow(y[17], 2) + 9.9999999999999995e-7) + 0.40824829046386307*u[2]*u[0]*cos(u[5] + u[7] - 2.0943951023931953); 
data[1126] = 2*p[7]*y[18] + p[8]*y[18]/sqrt(pow(y[25], 2) + pow(y[18], 2) + 9.9999999999999995e-7) + 0.40824829046386307*u[3]*u[0]*cos(u[5] + u[8] - 4.1887902047863905) + 0.40824829046386307*u[4]*u[0]*cos(u[9]); 
data[1127] = 2*p[7]*y[19] + p[8]*y[19]/sqrt(pow(y[26], 2) + pow(y[19], 2) + 9.9999999999999995e-7); 
data[1131] = 2*p[7]*y[23] + p[8]*y[23]/sqrt(pow(y[23], 2) + pow(y[16], 2) + 9.9999999999999995e-7) + 0.40824829046386307*u[1]*u[0]*sin(u[5] + u[6]); 
data[1132] = 2*p[7]*y[24] + p[8]*y[24]/sqrt(pow(y[24], 2) + pow(y[17], 2) + 9.9999999999999995e-7) + 0.40824829046386307*u[2]*u[0]*sin(u[5] + u[7] - 2.0943951023931953); 
data[1133] = 2*p[7]*y[25] + p[8]*y[25]/sqrt(pow(y[25], 2) + pow(y[18], 2) + 9.9999999999999995e-7) + 0.40824829046386307*u[3]*u[0]*sin(u[5] + u[8] - 4.1887902047863905) + 0.40824829046386307*u[4]*u[0]*sin(u[9]); 
data[1134] = 2*p[7]*y[26] + p[8]*y[26]/sqrt(pow(y[26], 2) + pow(y[19], 2) + 9.9999999999999995e-7); 
data[1190] = 0.33333333333333331*y[16] - 0.28867513459481287*y[24] - 0.16666666666666657*y[17] + 0.28867513459481275*y[25] - 0.16666666666666677*y[18]; 
data[1191] = 0.33333333333333331*y[23] - 0.16666666666666657*y[24] + 0.28867513459481287*y[17] - 0.16666666666666677*y[25] - 0.28867513459481275*y[18]; 
data[1192] = 0.28867513459481287*y[23] - 0.16666666666666657*y[16] + 0.33333333333333326*y[17] - 0.28867513459481281*y[25] - 0.16666666666666652*y[18]; 
data[1193] = -0.16666666666666657*y[23] - 0.28867513459481287*y[16] + 0.33333333333333326*y[24] - 0.16666666666666652*y[25] + 0.28867513459481281*y[18]; 
data[1194] = -0.28867513459481275*y[23] - 0.16666666666666677*y[16] + 0.28867513459481281*y[24] - 0.16666666666666652*y[17] + 0.33333333333333326*y[18]; 
data[1195] = -0.16666666666666677*y[23] + 0.28867513459481275*y[16] - 0.16666666666666652*y[24] - 0.28867513459481281*y[17] + 0.33333333333333326*y[25]; 
data[1206] = 0.33333333333333331*y[0] - 0.28867513459481287*y[3] - 0.16666666666666657*y[2] + 0.28867513459481275*y[5] - 0.16666666666666677*y[4]; 
data[1207] = 0.28867513459481287*y[1] - 0.16666666666666657*y[0] + 0.33333333333333326*y[2] - 0.28867513459481281*y[5] - 0.16666666666666652*y[4]; 
data[1208] = -0.28867513459481275*y[1] - 0.16666666666666677*y[0] + 0.28867513459481281*y[3] - 0.16666666666666652*y[2] + 0.33333333333333326*y[4]; 
data[1213] = 0.33333333333333331*y[1] - 0.16666666666666657*y[3] + 0.28867513459481287*y[2] - 0.16666666666666677*y[5] - 0.28867513459481275*y[4]; 
data[1214] = -0.16666666666666657*y[1] - 0.28867513459481287*y[0] + 0.33333333333333326*y[3] - 0.16666666666666652*y[5] + 0.28867513459481281*y[4]; 
data[1215] = -0.16666666666666677*y[1] + 0.28867513459481275*y[0] - 0.16666666666666652*y[3] - 0.28867513459481281*y[2] + 0.33333333333333326*y[5]; 
data[1231] = -0.33333333333333331*y[23] + 0.16666666666666657*y[24] - 0.28867513459481287*y[17] + 0.16666666666666677*y[25] + 0.28867513459481275*y[18]; 
data[1232] = 0.33333333333333331*y[16] - 0.28867513459481287*y[24] - 0.16666666666666657*y[17] + 0.28867513459481275*y[25] - 0.16666666666666677*y[18]; 
data[1233] = 0.16666666666666657*y[23] + 0.28867513459481287*y[16] - 0.33333333333333326*y[24] + 0.16666666666666652*y[25] - 0.28867513459481281*y[18]; 
data[1234] = 0.28867513459481287*y[23] - 0.16666666666666657*y[16] + 0.33333333333333326*y[17] - 0.28867513459481281*y[25] - 0.16666666666666652*y[18]; 
data[1235] = 0.16666666666666677*y[23] - 0.28867513459481275*y[16] + 0.16666666666666652*y[24] + 0.28867513459481281*y[17] - 0.33333333333333326*y[25]; 
data[1236] = -0.28867513459481275*y[23] - 0.16666666666666677*y[16] + 0.28867513459481281*y[24] - 0.16666666666666652*y[17] + 0.33333333333333326*y[18]; 
data[1247] = 0.33333333333333331*y[1] - 0.16666666666666657*y[3] + 0.28867513459481287*y[2] - 0.16666666666666677*y[5] - 0.28867513459481275*y[4]; 
data[1248] = -0.16666666666666657*y[1] - 0.28867513459481287*y[0] + 0.33333333333333326*y[3] - 0.16666666666666652*y[5] + 0.28867513459481281*y[4]; 
data[1249] = -0.16666666666666677*y[1] + 0.28867513459481275*y[0] - 0.16666666666666652*y[3] - 0.28867513459481281*y[2] + 0.33333333333333326*y[5]; 
data[1254] = -0.33333333333333331*y[0] + 0.28867513459481287*y[3] + 0.16666666666666657*y[2] - 0.28867513459481275*y[5] + 0.16666666666666677*y[4]; 
data[1255] = -0.28867513459481287*y[1] + 0.16666666666666657*y[0] - 0.33333333333333326*y[2] + 0.28867513459481281*y[5] + 0.16666666666666652*y[4]; 
data[1256] = 0.28867513459481275*y[1] + 0.16666666666666677*y[0] - 0.28867513459481281*y[3] + 0.16666666666666652*y[2] - 0.33333333333333326*y[4]; 

}

void de_jac_trap_num_eval(double *data,double *x,double *y,double *u,double *p,double Dt){

data[42] = 184.75059629806532; 
data[43] = 94.615102344455565; 
data[44] = -69.368640085185334; 
data[45] = -17.974903844364775; 
data[46] = -43.675799641021136; 
data[47] = -40.74868784166955; 
data[48] = -69.368640085185348; 
data[49] = -17.974903844364817; 
data[50] = -184.75059629806532; 
data[51] = -94.615102344455565; 
data[52] = 69.368640085185334; 
data[53] = 17.974903844364775; 
data[54] = 43.675799641021136; 
data[55] = 40.74868784166955; 
data[56] = 69.368640085185348; 
data[57] = 17.974903844364817; 
data[58] = -1; 
data[83] = -94.615102344455565; 
data[84] = 184.75059629806532; 
data[85] = 17.974903844364775; 
data[86] = -69.368640085185334; 
data[87] = 40.74868784166955; 
data[88] = -43.675799641021136; 
data[89] = 17.974903844364817; 
data[90] = -69.368640085185348; 
data[91] = 94.615102344455565; 
data[92] = -184.75059629806532; 
data[93] = -17.974903844364775; 
data[94] = 69.368640085185334; 
data[95] = -40.74868784166955; 
data[96] = 43.675799641021136; 
data[97] = -17.974903844364817; 
data[98] = 69.368640085185348; 
data[106] = -1; 
data[124] = -69.368640085185319; 
data[125] = -17.974903844364796; 
data[126] = 184.75059629806523; 
data[127] = 94.615102344455607; 
data[128] = -69.368640085185291; 
data[129] = -17.974903844364814; 
data[130] = -43.675799641021079; 
data[131] = -40.748687841669586; 
data[132] = 69.368640085185319; 
data[133] = 17.974903844364796; 
data[134] = -184.75059629806523; 
data[135] = -94.615102344455607; 
data[136] = 69.368640085185291; 
data[137] = 17.974903844364814; 
data[138] = 43.675799641021079; 
data[139] = 40.748687841669586; 
data[141] = -1; 
data[165] = 17.974903844364796; 
data[166] = -69.368640085185319; 
data[167] = -94.615102344455607; 
data[168] = 184.75059629806523; 
data[169] = 17.974903844364814; 
data[170] = -69.368640085185291; 
data[171] = 40.748687841669586; 
data[172] = -43.675799641021079; 
data[173] = -17.974903844364796; 
data[174] = 69.368640085185319; 
data[175] = 94.615102344455607; 
data[176] = -184.75059629806523; 
data[177] = -17.974903844364814; 
data[178] = 69.368640085185291; 
data[179] = -40.748687841669586; 
data[180] = 43.675799641021079; 
data[189] = -1; 
data[206] = -43.675799641021122; 
data[207] = -40.748687841669557; 
data[208] = -69.368640085185277; 
data[209] = -17.97490384436481; 
data[210] = 184.75059629806526; 
data[211] = 94.615102344455636; 
data[212] = -69.368640085185277; 
data[213] = -17.974903844364817; 
data[214] = 43.675799641021122; 
data[215] = 40.748687841669557; 
data[216] = 69.368640085185277; 
data[217] = 17.97490384436481; 
data[218] = -184.75059629806526; 
data[219] = -94.615102344455636; 
data[220] = 69.368640085185277; 
data[221] = 17.974903844364817; 
data[224] = -1; 
data[247] = 40.748687841669557; 
data[248] = -43.675799641021122; 
data[249] = 17.97490384436481; 
data[250] = -69.368640085185277; 
data[251] = -94.615102344455636; 
data[252] = 184.75059629806526; 
data[253] = 17.974903844364817; 
data[254] = -69.368640085185277; 
data[255] = -40.748687841669557; 
data[256] = 43.675799641021122; 
data[257] = -17.97490384436481; 
data[258] = 69.368640085185277; 
data[259] = 94.615102344455636; 
data[260] = -184.75059629806526; 
data[261] = -17.974903844364817; 
data[262] = 69.368640085185277; 
data[272] = -1; 
data[288] = -69.368640085185334; 
data[289] = -17.9749038443648; 
data[290] = -43.675799641021086; 
data[291] = -40.748687841669586; 
data[292] = -69.368640085185291; 
data[293] = -17.974903844364821; 
data[294] = 184.75059629806523; 
data[295] = 94.61510234445565; 
data[296] = 69.368640085185334; 
data[297] = 17.9749038443648; 
data[298] = 43.675799641021086; 
data[299] = 40.748687841669586; 
data[300] = 69.368640085185291; 
data[301] = 17.974903844364821; 
data[302] = -184.75059629806523; 
data[303] = -94.61510234445565; 
data[307] = -1; 
data[329] = 17.9749038443648; 
data[330] = -69.368640085185334; 
data[331] = 40.748687841669586; 
data[332] = -43.675799641021086; 
data[333] = 17.974903844364821; 
data[334] = -69.368640085185291; 
data[335] = -94.61510234445565; 
data[336] = 184.75059629806523; 
data[337] = -17.9749038443648; 
data[338] = 69.368640085185334; 
data[339] = -40.748687841669586; 
data[340] = 43.675799641021086; 
data[341] = -17.974903844364821; 
data[342] = 69.368640085185291; 
data[343] = 94.61510234445565; 
data[344] = -184.75059629806523; 
data[355] = -1; 
data[370] = -184.75059629806532; 
data[371] = -94.615102344455565; 
data[372] = 69.368640085185334; 
data[373] = 17.974903844364775; 
data[374] = 43.675799641021136; 
data[375] = 40.74868784166955; 
data[376] = 69.368640085185348; 
data[377] = 17.974903844364817; 
data[378] = 184.75059629806532; 
data[379] = 94.615102344455565; 
data[380] = -69.368640085185334; 
data[381] = -17.974903844364775; 
data[382] = -43.675799641021136; 
data[383] = -40.74868784166955; 
data[384] = -69.368640085185348; 
data[385] = -17.974903844364817; 
data[400] = -1; 
data[411] = 94.615102344455565; 
data[412] = -184.75059629806532; 
data[413] = -17.974903844364775; 
data[414] = 69.368640085185334; 
data[415] = -40.74868784166955; 
data[416] = 43.675799641021136; 
data[417] = -17.974903844364817; 
data[418] = 69.368640085185348; 
data[419] = -94.615102344455565; 
data[420] = 184.75059629806532; 
data[421] = 17.974903844364775; 
data[422] = -69.368640085185334; 
data[423] = 40.74868784166955; 
data[424] = -43.675799641021136; 
data[425] = 17.974903844364817; 
data[426] = -69.368640085185348; 
data[445] = -1; 
data[452] = 69.368640085185319; 
data[453] = 17.974903844364796; 
data[454] = -184.75059629806523; 
data[455] = -94.615102344455607; 
data[456] = 69.368640085185291; 
data[457] = 17.974903844364814; 
data[458] = 43.675799641021079; 
data[459] = 40.748687841669586; 
data[460] = -69.368640085185319; 
data[461] = -17.974903844364796; 
data[462] = 184.75059629806523; 
data[463] = 94.615102344455607; 
data[464] = -69.368640085185291; 
data[465] = -17.974903844364814; 
data[466] = -43.675799641021079; 
data[467] = -40.748687841669586; 
data[483] = -1; 
data[493] = -17.974903844364796; 
data[494] = 69.368640085185319; 
data[495] = 94.615102344455607; 
data[496] = -184.75059629806523; 
data[497] = -17.974903844364814; 
data[498] = 69.368640085185291; 
data[499] = -40.748687841669586; 
data[500] = 43.675799641021079; 
data[501] = 17.974903844364796; 
data[502] = -69.368640085185319; 
data[503] = -94.615102344455607; 
data[504] = 184.75059629806523; 
data[505] = 17.974903844364814; 
data[506] = -69.368640085185291; 
data[507] = 40.748687841669586; 
data[508] = -43.675799641021079; 
data[528] = -1; 
data[534] = 43.675799641021122; 
data[535] = 40.748687841669557; 
data[536] = 69.368640085185277; 
data[537] = 17.97490384436481; 
data[538] = -184.75059629806526; 
data[539] = -94.615102344455636; 
data[540] = 69.368640085185277; 
data[541] = 17.974903844364817; 
data[542] = -43.675799641021122; 
data[543] = -40.748687841669557; 
data[544] = -69.368640085185277; 
data[545] = -17.97490384436481; 
data[546] = 184.75059629806526; 
data[547] = 94.615102344455636; 
data[548] = -69.368640085185277; 
data[549] = -17.974903844364817; 
data[566] = -1; 
data[575] = -40.748687841669557; 
data[576] = 43.675799641021122; 
data[577] = -17.97490384436481; 
data[578] = 69.368640085185277; 
data[579] = 94.615102344455636; 
data[580] = -184.75059629806526; 
data[581] = -17.974903844364817; 
data[582] = 69.368640085185277; 
data[583] = 40.748687841669557; 
data[584] = -43.675799641021122; 
data[585] = 17.97490384436481; 
data[586] = -69.368640085185277; 
data[587] = -94.615102344455636; 
data[588] = 184.75059629806526; 
data[589] = 17.974903844364817; 
data[590] = -69.368640085185277; 
data[611] = -1; 
data[616] = 69.368640085185334; 
data[617] = 17.9749038443648; 
data[618] = 43.675799641021086; 
data[619] = 40.748687841669586; 
data[620] = 69.368640085185291; 
data[621] = 17.974903844364821; 
data[622] = -184.75059629806523; 
data[623] = -94.61510234445565; 
data[624] = -69.368640085185334; 
data[625] = -17.9749038443648; 
data[626] = -43.675799641021086; 
data[627] = -40.748687841669586; 
data[628] = -69.368640085185291; 
data[629] = -17.974903844364821; 
data[630] = 184.75059629806523; 
data[631] = 94.61510234445565; 
data[649] = -1; 
data[657] = -17.9749038443648; 
data[658] = 69.368640085185334; 
data[659] = -40.748687841669586; 
data[660] = 43.675799641021086; 
data[661] = -17.974903844364821; 
data[662] = 69.368640085185291; 
data[663] = 94.61510234445565; 
data[664] = -184.75059629806523; 
data[665] = 17.9749038443648; 
data[666] = -69.368640085185334; 
data[667] = 40.748687841669586; 
data[668] = -43.675799641021086; 
data[669] = 17.974903844364821; 
data[670] = -69.368640085185291; 
data[671] = -94.61510234445565; 
data[672] = 184.75059629806523; 
data[694] = -1; 
data[698] = -1; 
data[718] = 1; 
data[741] = -1; 
data[759] = 1; 
data[784] = -1; 
data[800] = 1; 
data[827] = -1; 
data[841] = 1; 
data[878] = 1; 
data[879] = 1; 
data[880] = 1; 
data[881] = 1; 
data[904] = -1.0; 
data[930] = 1.0; 
data[947] = -1.0; 
data[971] = 1.0; 
data[990] = -1.0; 
data[1012] = 1.0; 
data[1033] = -1.0; 
data[1053] = 1.0; 
data[1090] = 1.0; 
data[1091] = 1.0; 
data[1092] = 1.0; 
data[1093] = 1.0; 
data[1129] = -1; 
data[1171] = -1; 
data[1218] = -1; 
data[1260] = -1; 
data[1280] = -1; 
data[1323] = -1; 
data[1366] = -1; 
data[1409] = -1; 
data[1445] = -1.0; 
data[1488] = -1.0; 
data[1531] = -1.0; 
data[1574] = -1.0; 
data[1638] = -1; 
data[1680] = -1; 

}



/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // void()(double *, double *, double *, double *, double *, double)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 6), // double *
/*  2 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  3 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  4 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  5 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  6 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  8 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static void _cffi_d_de_jac_trap_num_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_trap_num_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_trap_num_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_trap_num_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_trap_num_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_trap_num_eval _cffi_d_de_jac_trap_num_eval
#endif

static void _cffi_d_de_jac_trap_up_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_trap_up_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_trap_up_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_trap_up_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_trap_up_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_trap_up_eval _cffi_d_de_jac_trap_up_eval
#endif

static void _cffi_d_de_jac_trap_xy_eval(double * x0, double * x1, double * x2, double * x3, double * x4, double x5)
{
  de_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_de_jac_trap_xy_eval(PyObject *self, PyObject *args)
{
  double * x0;
  double * x1;
  double * x2;
  double * x3;
  double * x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "de_jac_trap_xy_eval", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { de_jac_trap_xy_eval(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_de_jac_trap_xy_eval _cffi_d_de_jac_trap_xy_eval
#endif

static const struct _cffi_global_s _cffi_globals[] = {
  { "de_jac_trap_num_eval", (void *)_cffi_f_de_jac_trap_num_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_trap_num_eval },
  { "de_jac_trap_up_eval", (void *)_cffi_f_de_jac_trap_up_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_trap_up_eval },
  { "de_jac_trap_xy_eval", (void *)_cffi_f_de_jac_trap_xy_eval, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_de_jac_trap_xy_eval },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  NULL,  /* no fields */
  NULL,  /* no struct_unions */
  NULL,  /* no enums */
  NULL,  /* no typenames */
  3,  /* num_globals */
  0,  /* num_struct_unions */
  0,  /* num_enums */
  0,  /* num_typenames */
  NULL,  /* no includes */
  9,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit_temp_trap_cffi(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit_temp_trap_cffi(void) { return NULL; }
#  else
     inittemp_trap_cffi(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit_temp_trap_cffi(void)
{
  return _cffi_init("temp_trap_cffi", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
inittemp_trap_cffi(void)
{
  _cffi_init("temp_trap_cffi", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
