# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def add_model(grid,name,bus_name,data_dict):
    '''


    
    '''

    pass